/***************************************************************************
                          klcc.h  -  description
                             -------------------
    begin                : fre mar  9 20:53:57 CET 2001
    copyright            : (C) 2001 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KLCC_H
#define KLCC_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// Qt headers
#include <qwidget.h>
#include <qstrlist.h>
#include <qpixmap.h>
#include <qtimer.h>

// KDE headers
#include <kapp.h>
#include <kmainwindow.h>
#include <kaction.h>
#include <kconfig.h>
#include <kprocess.h>
#include <kdeversion.h>

// App headers
#include "klview.h"
#include "lcpio/lcpio.h"
#include "kldockwindow.h"
#include "kllinedlg.h"
#include "kldcop.h"

// ID for statusbar
#define ID_STATUS_MSG 1
#define ID_CONNECTION_MSG 2
#define ID_LINES_MSG 3

namespace KWallet
{
class Wallet;
}

class KLServerSettings;
class KLLookNFeelSettings;

/** KLCC is the base class of the project */
class KLCC : public KMainWindow {
  Q_OBJECT
  public:
    /** construtor */
    KLCC(QWidget*parent = 0, const char *name = 0);
    /** destructor */
    ~KLCC();
    /** get current line used */
    int currentLine();
  protected:
    /** init action */
    void initAction();
    /** init main view */
    void initView();
    /** init statusbar */
    void initStatusbar();
    /** init kicker system tray icon */
    void initDockWindow();
    /** save configuration */
    void saveOptions();
    /** load configuration */
    void loadOptions();
    /** queryExit is called by KMainWindow when the last window of the
     *  application is going to be closed during the closeEvent().
     *  Against the default implementation that just returns true, this calls
     *  saveOptions() to save the settings of the last window's
     *  properties.
     *  @see KMainWindow#queryExit
     *  @see KMainWindow#closeEvent
     */
    virtual bool queryExit();

    void connectWithPasswordRequest();

  private:
    /** main view */
    KLView* view;
    /** kicker menu */
    KLDockWindow* trayIcon;
    /** connect action */
    KAction* connectAction;
    /** disconnect action */
    KAction* disconnectAction;
    /** online action */
    KAction* onlineAction;
    /** offline action */
    KAction* offlineAction;
    /** disable auto-offline action */
    KToggleAction* disableAutoOfflineAction;
    /** line select action */
    KAction* selectAction;
    /** lock/unlock action */
    KAction* lockAction;
    /** serverstatus action */
    KAction* serverstatusAction;
    /** shutdown server action*/
    KAction* shutdownAction;
    /** Change num of used channels action */
    KAction* channelAction;
    /** toggle toolbar on/off */
    //KToggleAction* showToolbarAction;
    KToggleAction *menubarAction;
    /** toggle statusbar on/off */
    //KToggleAction* showStatusbarAction;
    /** show preferences dialog */
    KAction* preferencesAction;
    /** connection to server */
    LcpIO* lcpIO;
    /** configuration data */
    KConfig* config;
    // config vars
    // Server //
    KLServerSettings* serverSettings;
    // Look & Feel //
    KLLookNFeelSettings* lookNFeelSettings;
    /** timer needed for the beep interval */
    QTimer* beepTimer;
    /** DCOP interface */
    KLDcop* dcop;
    /** online/offlinescript */
    KShellProcess m_script;
    bool mExiting;
    KWallet::Wallet *mWallet;

  public slots:
    /**
     * enable/disable actions after clientstatus
     * @param status new status
     */
    void slotClientstatus(int status);
    /**
     * handle lock action after linestatus
     * @param status new status
     */
    void slotLinestatus(int status);
    /** connect to server */
    void slotConnect();
    /** disconnect from server */
    void slotDisconnect();
    /** ask server to go online */
    void slotOnline();
    /** ask server to go offline */
    void slotOffline();
    /** ask server to lock/unlock line as online or offline*/
    void slotLock();
    /**
     * select line to use
     * @param lines available lines
     */
    void slotLineSelection();
    /** setup KLCC */
    void slotSettings();

    void showMenubar();

    /**
     * changes the statusbar contents for the standard label permanently,
     * used to indicate current actions.
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusMsg(const QString &text, int id = ID_STATUS_MSG);
    /** connection lost to server */
    void slotConnectionLost();
    /** authentication failed */
    void slotBadAuth();
    /** handles access errors */
    void slotNoAccess(int cmd);
    /** handles channel operation failure */
    void slotChanFail();
    /** handle incomming call */
    void slotIsdnAct(t_lcp3_info_isdn info);
    /** view serverstatus */
    void slotServerstatus(t_lcp3_info_status status);
    /** changed num of channels used */
    void slotChannels(int num);
    /** Here comes a beep :) */
    void beep();
    /** toggle num of used isdn-channels */
    void slotToggleChan();
    /** set num of lines that are online on the server */
    void setNumLinesOnline(int lines);
    /** change settings when a linechange happens */
    void changeLineSettings(const QString& name);
    /** Notify when offline */
    void slotOfflineNotify();
    /** toggle disable/enable auto-offline */
    void slotAutoOfflineDisabled();
    /** configure key bindings */
    void slotKeyBindings();
    /** configure toolbars */
    void slotConfigToolbars();
    void slotNewToolbarConfig();
    /** configure notifications */
    void slotConfigNotifications();

    void quitKLCC();
  protected slots:
    void slotOfflineScriptReady();
  private slots:
    void closeEvent(QCloseEvent* e);
    void slotCloseWallet();
    void slotConnectWithWallet();
};

#endif
