/***************************************************************************
                          kldockwindow.cpp  -  description
                             -------------------
    begin                : Tue Mar 13 2001
    copyright            : (C) 2001 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qtooltip.h>
#include <qimage.h>

// KDE includes
#include <klocale.h>
#include <kiconloader.h>
#include <kiconeffect.h>

// App includes
#include "kldockwindow.h"
#include "lcpio/misc.h"

KLDockWindow::KLDockWindow(QWidget*parent, const char *name)
 : KSystemTray(parent, name)
{
	initDockWindow();
}

KLDockWindow::~KLDockWindow()
{
}

QPixmap KLDockWindow::renderIcon(const QString& baseIcon, const QString &overlayIcon) const
{
	if (overlayIcon.isNull())
		return KSystemTray::loadIcon(baseIcon);

	QImage baseImg = KSystemTray::loadIcon(baseIcon).convertToImage();
	QImage overlayImg = KSystemTray::loadIcon(overlayIcon).convertToImage();
	KIconEffect::overlay(baseImg, overlayImg);

	QPixmap base;
	base.convertFromImage(baseImg);
	return base;
}

void KLDockWindow::setStatus(int clientStatus, int lineStatus, int numChannels)
{
	if (mCurClientStatus != clientStatus || mCurLineStatus != lineStatus ||
		mNumChannels != numChannels)
	{
		QString base, overlay;
		mCurClientStatus = clientStatus;
		mCurLineStatus = lineStatus;
		mNumChannels = numChannels;

		switch(mCurClientStatus)
		{
		case CLT_DISCON:
		case CLT_NOAUTH:
		case CLT_CLOSING:
		case CLT_OFFLINE:
			base = "connect_no";
			break;
		case CLT_ONLINE:
			base = (numChannels == 2 ? "established2" : "connect_established");
			break;
		case CLT_DIALING:
		case CLT_ABORTING:
			base = "connect_creating";
			break;
		}

		switch(mCurLineStatus)
		{
		case CST_DOWN:
		case CST_CLOSING:
		case CST_CLOSE_ERROR:
		case CST_LOCKED_DOWN:
			overlay = "red_overlay";
			break;
		case CST_DIALING:
			overlay = "yellow_overlay";
			break;
		case CST_UP_USER:
			overlay = "blue_overlay";
			break;
		case CST_UP_SERVER:
		case CST_LOCKED_UP_CLT:
		case CST_LOCKED_UP_TIMED:
			overlay = "green_overlay";
			break;
		}

		QPixmap p = renderIcon(base, overlay);
		setPixmap(p);
	}
}

void KLDockWindow::initDockWindow()
{
	mCurClientStatus = -1;
	mCurLineStatus = -1;
	mNumChannels = 0;
	setStatus(CLT_DISCON, CST_DOWN, 0);
}

void KLDockWindow::setTooltip(t_lcp3_info_tput info)
{
	QToolTip::add(this, i18n("<qt><nobr>Client Time: %1</nobr><br><nobr>Line Time: %2</nobr></qt>")
		.arg(LCPIO::formatTime(info.client),LCPIO::formatTime(info.line)));
}

void KLDockWindow::slotClientstatus(int status)
{
	setStatus(status, mCurLineStatus, mNumChannels);
}

void KLDockWindow::slotLinestatus(int status)
{
	setStatus(mCurClientStatus, status, mNumChannels);
}

void KLDockWindow::slotChannels(int num)
{
	setStatus(mCurClientStatus, mCurLineStatus, num);
}

KPopupMenu* KLDockWindow::menu()
{
	return contextMenu();
}

#include "kldockwindow.moc"
