/***************************************************************************
                          kldockwindow.h  -  description
                             -------------------
    begin                : Tue Mar 13 2001
    copyright            : (C) 2001 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KLDOCKWINDOW_H
#define KLDOCKWINDOW_H

// Qt headers
#include <qpixmap.h>
#include <qwidget.h>

// KDE headers
#include <ksystemtray.h>
#include <kpopupmenu.h>

// app heanders
#include "lcpio/lcp3.h"

/**
 * Create the systemtray icon
 * @author Peter Simonsson
 */

class KLDockWindow : public KSystemTray
{
Q_OBJECT
public:
	KLDockWindow(QWidget*parent = 0, const char *name = 0);
	~KLDockWindow();
	KPopupMenu* menu();

protected:
	void initDockWindow();
	void setStatus(const QColor & status);

	QPixmap renderIcon(const QString& baseIcon, const QString &overlayIcon) const;
	void setStatus(int clientStatus, int lineStatus, int numChannels);

private:
	// last known client- and line-status
	// used to avoid unneeded icon re-rendering
	int mCurClientStatus;
	int mCurLineStatus;
	int mNumChannels;

public slots:
	/**
	 * Change the context menu after clientstatus
	 * @param status the new clientstatus
	 **/
	void slotClientstatus(int status);
	/**
	 * Update tooltip with lineinfo
	 * @param info lineinfo from server
	 **/
	void setTooltip(t_lcp3_info_tput info);
	/**
	 * Change the background after linestatus
	 * @param status the new linestatus
	 **/
	void slotLinestatus(int status);
	/**
	 * Change num isdn-channels in use
	 * @param num number of channels
	 **/
	void slotChannels(int num);
};

#endif
