/***************************************************************************
                          klpreferencesdlg.cpp  -  description
                             -------------------
    begin                : mn mar 4 2002
    copyright            : (C) 2002 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qframe.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qtabwidget.h>

// KDE includes
#include <klocale.h>
#include <kiconloader.h>
#include <kglobal.h>

// app includes
#include "klpreferencesdlg.h"
#include "klserversettingswidget.h"
#include "kllooknfeelsettingswidget.h"
#include "kllinesettingswidget.h"
#include "lcpio/lcpline.h"
#include "lcpio/misc.h"
#include "klsettings.h"

KLPreferencesDlg::KLPreferencesDlg(QWidget *parent, const char *name)
  : KDialogBase(IconList, i18n("Preferences"), Help|Ok|Cancel|Default,
  Ok, parent, name, true)
{
  m_currentLine = 0;
  // Server Page
  m_serverPage = addPage(i18n("Server"), i18n("LineControl Server Connection"),
    KGlobal::iconLoader()->loadIcon("network_local", KIcon::NoGroup, 32));
  QVBoxLayout* serverLayout = new QVBoxLayout(m_serverPage);
  m_serverWidget = new KLServerSettingsWidget(m_serverPage, "serverSettings");
  serverLayout->addWidget(m_serverWidget);
  // Look & Feel Page
  m_lookNFeelPage = addPage(i18n("Look & Feel"), i18n("Look & Feel"),
    KGlobal::iconLoader()->loadIcon("looknfeel", KIcon::NoGroup, 32));
  QVBoxLayout* lookNFeelLayout = new QVBoxLayout(m_lookNFeelPage);
  m_lookNFeelWidget = new KLLookNFeelSettingsWidget(m_lookNFeelPage, "lookNFeelSettings");
  lookNFeelLayout->addWidget(m_lookNFeelWidget);
  // Line Page
  m_linePage = addPage(i18n("Line"), i18n("LineControl Lines"),
    KGlobal::iconLoader()->loadIcon("connect_established", KIcon::NoGroup, 32));
  QVBoxLayout* lineLayout = new QVBoxLayout(m_linePage);
  m_lineWidget = new KLLineSettingsWidget(m_linePage, "lineSettings");
  lineLayout->addWidget(m_lineWidget);
  connect(m_lineWidget->lineCmBox, SIGNAL(activated(int)), this, SLOT(changeLineDisp(int)));
  connect(m_lineWidget->lineCmBox, SIGNAL(activated(int)), this, SLOT(changeLineUI()));
  connect(m_lineWidget->standardCBox, SIGNAL(clicked()), this, SLOT(changeLineUI()));
}

KLPreferencesDlg::~KLPreferencesDlg() {
}

void KLPreferencesDlg::slotDefault() {
  KLocale *loc = KGlobal::locale();

  if(activePageIndex() == pageIndex(m_serverPage)) {
    // Set defaults for serversettings
    m_serverWidget->hostEdit->setText("localhost");
    m_serverWidget->portEdit->setText("16007");
    m_serverWidget->userEdit->setText("");
    //m_serverWidget->passwdEdit->setText("");
    m_serverWidget->savePasswdCBox->setChecked(false);
    m_serverWidget->useAuthCBox->setChecked(false);
    m_serverWidget->autoConnectCBox->setChecked(false);
    m_serverWidget->reconnectCBox->setChecked(false);
    m_serverWidget->reconnectDelaySBox->setValue(0);
    m_serverWidget->reconnectCountSBox->setValue(0);
  } else if(activePageIndex() == pageIndex(m_lookNFeelPage)) {
    m_lookNFeelWidget->onlineCBox->setChecked(false);
    m_lookNFeelWidget->dockCBox->setChecked(true);
    m_lookNFeelWidget->minimizeCBox->setChecked(false);
    m_lookNFeelWidget->winStateCBox->setChecked(false);
    m_lookNFeelWidget->winPositionCBox->setChecked(false);
    m_lookNFeelWidget->lastLineCBox->setChecked(false);
    m_lookNFeelWidget->redialCBox->setChecked(false);
    m_lookNFeelWidget->redialCountSBox->setValue(3);
    m_lookNFeelWidget->redialDelaySBox->setValue(3);
    m_lookNFeelWidget->onlineWarningSBox->setValue(2);
  } else if(activePageIndex() == pageIndex(m_linePage)) {
    m_lineWidget->standardCBox->setChecked(true);
    m_lineWidget->autoOfflineCBox->setChecked(false);
    m_lineWidget->costCBox->setChecked(false);
    m_lineWidget->channelCBox->setChecked(false);
    m_lineWidget->scriptOnlineCBox->setChecked(false);
    m_lineWidget->scriptOfflineCBox->setChecked(false);
    m_lineWidget->scriptOnlineEdit->setText("");
    m_lineWidget->scriptOfflineEdit->setText("");
    m_lineWidget->logicOrRBtn->setChecked(true);
    m_lineWidget->activityCBox->setChecked(false);
    m_lineWidget->offlineTimeCBox->setChecked(false);
    m_lineWidget->offlineDownCBox->setChecked(false);
    m_lineWidget->offlineUpCBox->setChecked(false);
    m_lineWidget->offlineTotalDownCBox->setChecked(false);
    m_lineWidget->offlineTotalUpCBox->setChecked(false);
    m_lineWidget->offlineTimeSBox->setValue(60);
    m_lineWidget->offlineDownSBox->setValue(1000);
    m_lineWidget->offlineUpSBox->setValue(100);
    m_lineWidget->offlineTotalDownSBox->setValue(20);
    m_lineWidget->offlineTotalUpSBox->setValue(2);
    m_lineWidget->costEdit->setText(loc->formatNumber(0, 3));
     m_lineWidget->connectionFeeEdit->setText(loc->formatNumber(0, 3));

    for(uint i = 0; i < m_lineList.count(); i++) {
      saveValuesToLine(i);
    }
  }
}

void KLPreferencesDlg::addLine(LcpLine line) {
  ListPost* post = new ListPost;
  post->useDefault = line.useDefaultSettings();
  post->offlineInfo = line.offlineInfo();
  post->costInfo = line.costInfo();
  post->useTwoChan = line.useTwoChanDefault();
  post->onlineScriptActive = line.onlineScriptActive();
  post->offlineScriptActive = line.offlineScriptActive();
  post->onlineScript = line.onlineScript();
  post->offlineScript = line.offlineScript();
  m_lineList.append(post);

  if(m_lineList.count() == 1) {
    m_lineWidget->lineCmBox->insertItem(i18n("Default"));
    loadValuesFromLine(0);
  } else {
    m_lineWidget->lineCmBox->insertItem(line.name());
  }
}

void KLPreferencesDlg::getLine(int num, LcpLine& line) {
  if(num == m_currentLine) {
    saveValuesToLine(m_currentLine);
  }

  line.setUseDefaultSettings(m_lineList.at(num)->useDefault);
  line.setOfflineInfo(m_lineList.at(num)->offlineInfo);
  line.setCostInfo(m_lineList.at(num)->costInfo);
  line.setUseTwoChanDefault(m_lineList.at(num)->useTwoChan);
  line.setOnlineScriptActive(m_lineList.at(num)->onlineScriptActive);
  line.setOfflineScriptActive(m_lineList.at(num)->offlineScriptActive);
  line.setOnlineScript(m_lineList.at(num)->onlineScript);
  line.setOfflineScript(m_lineList.at(num)->offlineScript);
}

void KLPreferencesDlg::changeLineDisp(int line) {
  saveValuesToLine(m_currentLine);
  m_currentLine = line;
  loadValuesFromLine(line);
}

void KLPreferencesDlg::saveValuesToLine(int line) {
  KLocale *loc = KGlobal::locale();

  m_lineList.at(line)->useDefault = m_lineWidget->standardCBox->isChecked();

  if(m_lineWidget->standardCBox->isChecked() && line != 0) {
    m_lineList.at(line)->costInfo = m_lineList.at(0)->costInfo;
    m_lineList.at(line)->offlineInfo = m_lineList.at(0)->offlineInfo;
    m_lineList.at(line)->useTwoChan = m_lineList.at(0)->useTwoChan;
    m_lineList.at(line)->onlineScriptActive = m_lineList.at(0)->onlineScriptActive;
    m_lineList.at(line)->offlineScriptActive = m_lineList.at(0)->offlineScriptActive;
    m_lineList.at(line)->onlineScript = m_lineList.at(0)->onlineScript;
    m_lineList.at(line)->offlineScript = m_lineList.at(0)->offlineScript;
  } else {
    m_lineList.at(line)->offlineInfo.autoOfflineActive = m_lineWidget->autoOfflineCBox->isChecked();
    m_lineList.at(line)->costInfo.costCalcActive = m_lineWidget->costCBox->isChecked();
    m_lineList.at(line)->useTwoChan = m_lineWidget->channelCBox->isChecked();

    if(m_lineWidget->logicOrRBtn->isChecked()) {
      m_lineList.at(line)->offlineInfo.logic = LCPIO::OR;
    } else {
      m_lineList.at(line)->offlineInfo.logic = LCPIO::AND;
    }

    if(m_lineWidget->activityCBox->isChecked()) {
      m_lineList.at(line)->offlineInfo.mode = LCPIO::Activity;
    } else {
      m_lineList.at(line)->offlineInfo.mode = LCPIO::Normal;
    }

    m_lineList.at(line)->offlineInfo.timeActive = m_lineWidget->offlineTimeCBox->isChecked();
    m_lineList.at(line)->offlineInfo.downloadActive = m_lineWidget->offlineDownCBox->isChecked();
    m_lineList.at(line)->offlineInfo.uploadActive = m_lineWidget->offlineUpCBox->isChecked();
    m_lineList.at(line)->offlineInfo.maxDownloadActive = m_lineWidget->offlineTotalDownCBox->isChecked();
    m_lineList.at(line)->offlineInfo.maxUploadActive = m_lineWidget->offlineTotalUpCBox->isChecked();
    m_lineList.at(line)->offlineInfo.time = m_lineWidget->offlineTimeSBox->value();
    m_lineList.at(line)->offlineInfo.download = m_lineWidget->offlineDownSBox->value();
    m_lineList.at(line)->offlineInfo.upload = m_lineWidget->offlineUpSBox->value();
    m_lineList.at(line)->offlineInfo.maxDownload = m_lineWidget->offlineTotalDownSBox->value() * 1024;
    m_lineList.at(line)->offlineInfo.maxUpload = m_lineWidget->offlineTotalUpSBox->value() * 1024;
    loc->readNumber(m_lineWidget->costEdit->text());
    m_lineList.at(line)->costInfo.cost = loc->readNumber(m_lineWidget->costEdit->text());
    m_lineList.at(line)->costInfo.connectionFee = loc->readNumber(m_lineWidget->connectionFeeEdit->text());
    m_lineList.at(line)->useTwoChan = m_lineWidget->channelCBox->isChecked();
    m_lineList.at(line)->onlineScriptActive = m_lineWidget->scriptOnlineCBox->isChecked();
    m_lineList.at(line)->offlineScriptActive = m_lineWidget->scriptOfflineCBox->isChecked();
    m_lineList.at(line)->onlineScript = m_lineWidget->scriptOnlineEdit->text();
    m_lineList.at(line)->offlineScript = m_lineWidget->scriptOfflineEdit->text();
  }
}

void KLPreferencesDlg::loadValuesFromLine(int line) {
  KLocale *loc = KGlobal::locale();

  m_lineWidget->standardCBox->setChecked(m_lineList.at(line)->useDefault);
  m_lineWidget->autoOfflineCBox->setChecked(m_lineList.at(line)->offlineInfo.autoOfflineActive);
  m_lineWidget->costCBox->setChecked(m_lineList.at(line)->costInfo.costCalcActive);
  m_lineWidget->channelCBox->setChecked(m_lineList.at(line)->useTwoChan);
  m_lineWidget->scriptOnlineCBox->setChecked(m_lineList.at(line)->onlineScriptActive);
  m_lineWidget->scriptOfflineCBox->setChecked(m_lineList.at(line)->offlineScriptActive);
  m_lineWidget->scriptOnlineEdit->setText(m_lineList.at(line)->onlineScript);
  m_lineWidget->scriptOfflineEdit->setText(m_lineList.at(line)->offlineScript);

  if(m_lineList.at(line)->offlineInfo.logic == LCPIO::OR) {
    m_lineWidget->logicOrRBtn->setChecked(true);
  } else if(m_lineList.at(line)->offlineInfo.logic == LCPIO::AND) {
    m_lineWidget->logicAndRBtn->setChecked(true);
  }

  if(m_lineList.at(line)->offlineInfo.mode == LCPIO::Normal) {
    m_lineWidget->activityCBox->setChecked(false);
  } else if(m_lineList.at(line)->offlineInfo.mode == LCPIO::Activity) {
    m_lineWidget->activityCBox->setChecked(true);
  }

  m_lineWidget->offlineTimeCBox->setChecked(m_lineList.at(line)->offlineInfo.timeActive);
  m_lineWidget->offlineDownCBox->setChecked(m_lineList.at(line)->offlineInfo.downloadActive);
  m_lineWidget->offlineUpCBox->setChecked(m_lineList.at(line)->offlineInfo.uploadActive);
  m_lineWidget->offlineTotalDownCBox->setChecked(m_lineList.at(line)->offlineInfo.maxDownloadActive);
  m_lineWidget->offlineTotalUpCBox->setChecked(m_lineList.at(line)->offlineInfo.maxUploadActive);
  m_lineWidget->offlineTimeSBox->setValue(m_lineList.at(line)->offlineInfo.time);
  m_lineWidget->offlineDownSBox->setValue(m_lineList.at(line)->offlineInfo.download);
  m_lineWidget->offlineUpSBox->setValue(m_lineList.at(line)->offlineInfo.upload);
  m_lineWidget->offlineTotalDownSBox->setValue(m_lineList.at(line)->offlineInfo.maxDownload / 1024);
  m_lineWidget->offlineTotalUpSBox->setValue(m_lineList.at(line)->offlineInfo.maxUpload / 1024);
  m_lineWidget->costEdit->setText(loc->formatNumber(m_lineList.at(line)->costInfo.cost, 3));
  m_lineWidget->connectionFeeEdit->setText(loc->formatNumber(m_lineList.at(line)->costInfo.connectionFee, 3));
}

void KLPreferencesDlg::changeLineUI() {
  m_lineWidget->standardCBox->setEnabled(m_lineWidget->lineCmBox->currentItem() != 0);

  bool allowChange = m_lineWidget->lineCmBox->currentItem() == 0 || \
    (m_lineWidget->lineCmBox->currentItem() != 0 &&
    !m_lineWidget->standardCBox->isChecked());

  m_lineWidget->settingsTab->setEnabled(allowChange);
}

void KLPreferencesDlg::addServerSettings(KLServerSettings settings) {
  m_serverWidget->hostEdit->setText(settings.host());
  m_serverWidget->portEdit->setText(QString::number(settings.port()));
  m_serverWidget->userEdit->setText(settings.user());
  //m_serverWidget->passwdEdit->setText(settings.passwd());
  m_serverWidget->savePasswdCBox->setChecked(settings.savePasswd());
  m_serverWidget->useAuthCBox->setChecked(settings.authenticate());
  m_serverWidget->autoConnectCBox->setChecked(settings.connectOnStartup());
  m_serverWidget->reconnectCBox->setChecked(settings.reconnect());
  m_serverWidget->reconnectDelaySBox->setValue(settings.reconnectDelay());
  m_serverWidget->reconnectCountSBox->setValue(settings.reconnectCount());
}

void KLPreferencesDlg::addLookNFeelSettings(KLLookNFeelSettings settings) {
  m_lookNFeelWidget->onlineCBox->setChecked(settings.autoOnline());
  m_lookNFeelWidget->dockCBox->setChecked(settings.dock());
  m_lookNFeelWidget->minimizeCBox->setChecked(settings.minimizeOnline());
  m_lookNFeelWidget->winStateCBox->setChecked(settings.saveWinState());
  m_lookNFeelWidget->winPositionCBox->setChecked(settings.saveWinPos());
  m_lookNFeelWidget->lastLineCBox->setChecked(settings.saveLastLine());
  m_lookNFeelWidget->redialCBox->setChecked(settings.redial());
  m_lookNFeelWidget->redialCountSBox->setValue(settings.redialCount());
  m_lookNFeelWidget->redialDelaySBox->setValue(settings.redialDelay());
  m_lookNFeelWidget->onlineWarningSBox->setValue(settings.onlineWarningDelay());
}

void KLPreferencesDlg::getServerSettings(KLServerSettings& settings) {
  settings.setHost(m_serverWidget->hostEdit->text());
  settings.setPort(m_serverWidget->portEdit->text().toUInt());
  settings.setUser(m_serverWidget->userEdit->text());
  //settings.setPasswd(m_serverWidget->passwdEdit->text());
  settings.setSavePasswd(m_serverWidget->savePasswdCBox->isChecked());
  settings.setAuthenticate(m_serverWidget->useAuthCBox->isChecked());
  settings.setConnectOnStartup(m_serverWidget->autoConnectCBox->isChecked());
  settings.setReconnect(m_serverWidget->reconnectCBox->isChecked());
  settings.setReconnectDelay(m_serverWidget->reconnectDelaySBox->value());
  settings.setReconnectCount(m_serverWidget->reconnectCountSBox->value());
}

void KLPreferencesDlg::getLookNFeelSettings(KLLookNFeelSettings& settings) {
  settings.setAutoOnline(m_lookNFeelWidget->onlineCBox->isChecked());
  settings.setDock(m_lookNFeelWidget->dockCBox->isChecked());
  settings.setMinimizeOnline(m_lookNFeelWidget->minimizeCBox->isChecked());
  settings.setSaveWinState(m_lookNFeelWidget->winStateCBox->isChecked());
  settings.setSaveWinPos(m_lookNFeelWidget->winPositionCBox->isChecked());
  settings.setSaveLastLine(m_lookNFeelWidget->lastLineCBox->isChecked());
  settings.setRedial(m_lookNFeelWidget->redialCBox->isChecked());
  settings.setRedialCount(m_lookNFeelWidget->redialCountSBox->value());
  settings.setRedialDelay(m_lookNFeelWidget->redialDelaySBox->value());
  settings.setOnlineWarningDelay(m_lookNFeelWidget->onlineWarningSBox->value());
}

#include "klpreferencesdlg.moc"
