/***************************************************************************
                          klsettings.cpp  -  description
                             -------------------
    begin                : fre mar 8 2002
    copyright            : (C) 2002 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qstring.h>

// KDE includes
#include <kconfig.h>

// app includes
#include "klsettings.h"

KLServerSettings::KLServerSettings(){
}
KLServerSettings::~KLServerSettings(){
}

void KLServerSettings::saveSettings(KConfig* config) {
  config->setGroup("Server Options");
  config->writeEntry("Host", m_host);
  config->writeEntry("Port", m_port);
  config->writeEntry("User", m_user);
  config->writeEntry("Save Password", m_savePasswd);
  config->writeEntry("Use Authentication", m_authenticate);
  config->writeEntry("Use Auto-reconnect", m_reconnect);
  config->writeEntry("Max Reconnect Tries", m_reconnectCount);
  config->writeEntry("Reconnect After", m_reconnectDelay);

  config->setGroup("Client Options");
  config->writeEntry("Auto Connect", m_connectOnStartup);
}

void KLServerSettings::loadSettings(KConfig* config) {
  config->setGroup("Server Options");
  m_host = config->readEntry("Host", "localhost");
  m_port = config->readUnsignedNumEntry("Port", 16007);
  m_user = config->readEntry("User", "");
  m_savePasswd = config->readBoolEntry("Save Password", false);
  m_authenticate = config->readBoolEntry("Use Authentication", false);
  m_reconnect = config->readBoolEntry("Use Auto-reconnect", false);
  m_reconnectCount = config->readNumEntry("Max Reconnect Tries", 0);
  m_reconnectDelay = config->readNumEntry("Reconnect After", 0);

  config->setGroup("Client Options");
  m_connectOnStartup = config->readBoolEntry("Auto Connect", false);
}

void KLServerSettings::setHost(const QString host) {
  m_host = host;
}

void KLServerSettings::setPort(const uint port) {
  m_port = port;
}

void KLServerSettings::setUser(const QString user) {
  m_user = user;
}

void KLServerSettings::setAuthenticate(const bool auth) {
  m_authenticate = auth;
}

void KLServerSettings::setSavePasswd(const bool save) {
  m_savePasswd = save;
}

void KLServerSettings::setConnectOnStartup(const bool connect) {
  m_connectOnStartup = connect;
}

void KLServerSettings::setReconnect(const bool reconnect) {
  m_reconnect = reconnect;
}

void KLServerSettings::setReconnectDelay(const int delay) {
  m_reconnectDelay = delay;
}

void KLServerSettings::setReconnectCount(const int count) {
  m_reconnectCount = count;
}

QString KLServerSettings::host() {
  return m_host;
}

uint KLServerSettings::port() {
  return m_port;
}

QString KLServerSettings::user() {
  return m_user;
}

bool KLServerSettings::authenticate() {
  return m_authenticate;
}

bool KLServerSettings::savePasswd() {
  return m_savePasswd;
}

bool KLServerSettings::connectOnStartup() {
  return m_connectOnStartup;
}

bool KLServerSettings::reconnect() {
  return m_reconnect;
}

int KLServerSettings::reconnectDelay() {
  return m_reconnectDelay;
}

int KLServerSettings::reconnectCount() {
  return m_reconnectCount;
}

// KLLookNFeelSettings

KLLookNFeelSettings::KLLookNFeelSettings() {
}

KLLookNFeelSettings::~KLLookNFeelSettings() {
}

void KLLookNFeelSettings::saveSettings(KConfig* config) {
  config->setGroup("General Options");
  config->writeEntry("Save Position", m_saveWinPos);
  config->writeEntry("Save State", m_saveWinState);

  config->setGroup("Client Options");
  config->writeEntry("Auto Online", m_autoOnline);
  config->writeEntry("Dock", m_dock);
  config->writeEntry("Hide", m_minimizeOnline);
  config->writeEntry("Beep Period", m_onlineWarningDelay);
  config->writeEntry("Save Line", m_saveLastLine);
  config->writeEntry("Use AutoRedial", m_redial);
  config->writeEntry("Max Tries Redial", m_redialCount);
  config->writeEntry("Redial After", m_redialDelay);
}

void KLLookNFeelSettings::loadSettings(KConfig* config) {
  config->setGroup("General Options");
  m_saveWinPos = config->readBoolEntry("Save Position", false);
  m_saveWinState = config->readBoolEntry("Save State", false);

  config->setGroup("Client Options");
  m_autoOnline = config->readBoolEntry("Auto Online", false);
  m_dock = config->readBoolEntry("Dock", true);
  m_minimizeOnline = config->readBoolEntry("Hide", false);
  m_onlineWarningDelay = config->readNumEntry("Beep Period", 2);
  m_saveLastLine = config->readBoolEntry("Save Line", false);
  m_redial = config->readBoolEntry("Use AutoRedial", false);
  m_redialCount = config->readNumEntry("Max Tries Redial", 3);
  m_redialDelay = config->readNumEntry("Redial After", 3);
}

void KLLookNFeelSettings::setDock(bool dock) {
  m_dock = dock;
}

void KLLookNFeelSettings::setMinimizeOnline(bool minimize) {
  m_minimizeOnline = minimize;
}

void KLLookNFeelSettings::setSaveWinState(bool save) {
  m_saveWinState = save;
}

void KLLookNFeelSettings::setSaveWinPos(bool save) {
  m_saveWinPos = save;
}

void KLLookNFeelSettings::setSaveLastLine(bool save) {
  m_saveLastLine = save;
}

void KLLookNFeelSettings::setAutoOnline(bool online) {
  m_autoOnline = online;
}

void KLLookNFeelSettings::setOnlineWarningDelay(int delay) {
  m_onlineWarningDelay = delay;
}

void KLLookNFeelSettings::setRedial(bool redial) {
  m_redial = redial;
}

void KLLookNFeelSettings::setRedialCount(int count) {
  m_redialCount = count;
}

void KLLookNFeelSettings::setRedialDelay(int delay) {
  m_redialDelay = delay;
}

bool KLLookNFeelSettings::dock() {
  return m_dock;
}

bool KLLookNFeelSettings::minimizeOnline() {
  return m_minimizeOnline;
}

bool KLLookNFeelSettings::saveWinState() {
  return m_saveWinState;
}

bool KLLookNFeelSettings::saveWinPos() {
  return m_saveWinPos;
}

bool KLLookNFeelSettings::saveLastLine() {
  return m_saveLastLine;
}

bool KLLookNFeelSettings::autoOnline() {
  return m_autoOnline;
}

int KLLookNFeelSettings::onlineWarningDelay() {
  return m_onlineWarningDelay;
}

bool KLLookNFeelSettings::redial() {
  return m_redial;
}

int KLLookNFeelSettings::redialCount() {
  return m_redialCount;
}

int KLLookNFeelSettings::redialDelay() {
  return m_redialDelay;
}
