/***************************************************************************
                          klsettings.h  -  description
                             -------------------
    begin                : fre mar 8 2002
    copyright            : (C) 2002 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KLSETTINGS_H
#define KLSETTINGS_H

class QString;
class KConfig;

/**
 * Class that handels the serversettings
 * @author Peter Simonsson
 */

class KLServerSettings {
  public:
    KLServerSettings();
    ~KLServerSettings();
    void saveSettings(KConfig* config);
    void loadSettings(KConfig* config);
    void setHost(const QString host);
    void setPort(const uint port);
    void setUser(const QString user);
    void setAuthenticate(const bool auth);
    void setSavePasswd(const bool save);
    void setConnectOnStartup(const bool connect);
    void setReconnect(const bool reconnect);
    void setReconnectDelay(const int delay);
    void setReconnectCount(const int count);
    QString host();
    uint port();
    QString user();
    bool authenticate();
    bool savePasswd();
    bool connectOnStartup();
    bool reconnect();
    int reconnectDelay();
    int reconnectCount();
  protected: // Variables //
    QString m_host;
    uint m_port;
    QString m_user;
    bool m_authenticate;
    bool m_savePasswd;
    bool m_connectOnStartup;
    bool m_reconnect;
    int m_reconnectDelay;
    int m_reconnectCount;
};

/**
 * Class that handels the look & feel settings
 * @author Peter Simonsson
 */

class KLLookNFeelSettings {
  public:
    KLLookNFeelSettings();
    ~KLLookNFeelSettings();
    void saveSettings(KConfig* config);
    void loadSettings(KConfig* config);
    void setDock(bool dock);
    void setMinimizeOnline(bool minimize);
    void setSaveWinState(bool save);
    void setSaveWinPos(bool save);
    void setSaveLastLine(bool save);
    void setAutoOnline(bool online);
    void setOnlineWarningDelay(int delay);
    void setRedial(bool redial);
    void setRedialCount(int count);
    void setRedialDelay(int delay);
    bool dock();
    bool minimizeOnline();
    bool saveWinState();
    bool saveWinPos();
    bool saveLastLine();
    bool autoOnline();
    int onlineWarningDelay();
    bool redial();
    int redialCount();
    int redialDelay();
  protected: // Variables //
    bool m_dock;
    bool m_minimizeOnline;
    bool m_saveWinState;
    bool m_saveWinPos;
    bool m_saveLastLine;
    bool m_autoOnline;
    int m_onlineWarningDelay;
    bool m_redial;
    int m_redialCount;
    int m_redialDelay;
};

#endif
