/***************************************************************************
                          klview.cpp  -  description
                             -------------------
    begin                : Thu Mar 8 2001
    copyright            : (C) 2001 by
    email                :
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qlabel.h>

#include <kglobal.h>
#include <klocale.h>
#include <kio/global.h>

// App includes
#include "klview.h"
#include "lcpio/misc.h"

KLView::KLView(QWidget* parent, const char* name, WFlags fl)
	: KLMainView(parent, name, fl)
{
  locale = KGlobal::locale();
  doCalculateCost = false;
  twoChan = false;
  clCostLbl->setEnabled(doCalculateCost);
  liCostLbl->setEnabled(doCalculateCost);
  clCostTxtLbl->setEnabled(doCalculateCost);
  liCostTxtLbl->setEnabled(doCalculateCost);
  clCostLbl->setText(locale->formatMoney(0.0f));
  liCostLbl->setText(locale->formatMoney(0.0f));
}

KLView::~KLView()
{
}

void KLView::setCostCalculationActive(bool use) {
  doCalculateCost = use;
  clCostLbl->setEnabled(doCalculateCost);
  liCostLbl->setEnabled(doCalculateCost);
  clCostTxtLbl->setEnabled(doCalculateCost);
  liCostTxtLbl->setEnabled(doCalculateCost);
}

bool KLView::getCostCalculationActive() const {
  return doCalculateCost;
}

void KLView::setClientstatus(int status) {
  switch(status) {
    case CLT_DISCON:
      clStatusLbl->setText(i18n("Disconnected"));
      break;
    case CLT_NOAUTH:
      clStatusLbl->setText(i18n("Disconnected"));
      break;
    case CLT_OFFLINE:
      clStatusLbl->setText(i18n("Offline"));
      break;
    case CLT_DIALING:
      clStatusLbl->setText(i18n("Dialing..."));
      break;
    case CLT_ONLINE:
      clStatusLbl->setText(i18n("Online"));
      break;
    case CLT_CLOSING:
      clStatusLbl->setText(i18n("Closing..."));
      break;
    case CLT_ABORTING:
      clStatusLbl->setText(i18n("Aborting..."));
  }
}

void KLView::setLinestatus(int status) {
  QString isdnChan = i18n("[2 channels]");
  QString tmp;

  switch(status) {
    case CST_DOWN:
      liStatusLbl->setText(i18n("Offline"));
      break;
    case CST_DIALING:
      liStatusLbl->setText(i18n("Dialing..."));
      break;
    case CST_UP_SERVER:
      tmp = i18n("Online");

      if(twoChan) {
        tmp += " " + isdnChan;
      }

      liStatusLbl->setText(tmp);
      break;
    case CST_UP_USER:
      tmp = i18n("Online (user)");

      if(twoChan) {
        tmp += " " + isdnChan;
      }

      liStatusLbl->setText(tmp);
      break;
    case CST_CLOSING:
      liStatusLbl->setText(i18n("Closing..."));
      break;
    case CST_CLOSE_ERROR:
      liStatusLbl->setText(i18n("Error!"));
      break;
    case CST_LOCKED_UP_CLT:	// this line stays established!
      if(twoChan)
        liStatusLbl->setText(i18n("Online (Locked) %1").arg(isdnChan));
      else
        liStatusLbl->setText(i18n("Online (Locked)"));
      break;
    case CST_LOCKED_DOWN:	// this line stays down!
      liStatusLbl->setText(i18n("Offline (Locked)"));
      break;
    case CST_LOCKED_UP_TIMED:	// the line will be closed when some timer ends
      if(twoChan)
        liStatusLbl->setText(i18n("Online (Timelocked) %1").arg(isdnChan));
      else
        liStatusLbl->setText(i18n("Online (Timelocked)"));
      break;
  }
}

void KLView::setInfo(struct t_lcp3_info_tput info, double costClient, double costLine)
{
	liDownloadLbl->setText(i18n("123,4 B/s", "%1/s").arg(KIO::convertSize(info.dns)));
	liUploadLbl->setText(i18n("123,4 B/s", "%1/s").arg(KIO::convertSize(info.ups)));
	liDownTotalLbl->setText(KIO::convertSize(info.dnt * 1024));
	liUpTotalLbl->setText(KIO::convertSize(info.upt * 1024));

	clTimeLbl->setText(LCPIO::formatTime(info.client));
	liTimeLbl->setText(LCPIO::formatTime(info.line));

	if(doCalculateCost)
	{
		clCostLbl->setText(locale->formatMoney(costClient));
		liCostLbl->setText(locale->formatMoney(costLine));
	}
}

void KLView::setLinename(const QString &name) {
  liNameLbl->setText(name);
}

void KLView::setTwoChan(bool use) {
  QString isdnChan = i18n("[2 channels]");
  QString tmp;

  twoChan = use;

  if(twoChan) {
    tmp = liStatusLbl->text() + " " + isdnChan;
  } else {
    tmp = liStatusLbl->text();
    tmp.truncate(tmp.findRev(" " + isdnChan));
  }

  liStatusLbl->setText(tmp);
}

#include "klview.moc"
