/***************************************************************************
                          lcpline.cpp  -  description
                             -------------------
    begin                : Thu Oct 18 2001
    copyright            : (C) 2001 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qstring.h>

// KDE includes
#include <kconfig.h>

// app includes
#include "lcpline.h"
#include "lcp3.h"

// Define static variables
bool LcpLine::s_autoOfflineDisabled = false;
int LcpLine::s_numLinesOnline = 0;

LcpLine::LcpLine(const QString name, const QString host) {
  m_host = host;
  m_name = name;
  m_numChannels = 1; // one channel is standard
  m_activityTime = 0;
  m_status = CST_DOWN;
  loadSettings();
}

LcpLine::~LcpLine() {
  saveSettings();
  s_numLinesOnline = 0;
}

int LcpLine::status() const {
  return m_status;
}

uint LcpLine::maxThroughputUp() const {
  return m_maxThroughputUp;
}

uint LcpLine::maxThroughputDown() const {
  return m_maxThroughputDown;
}

int LcpLine::numChannels() const {
  return m_numChannels;
}

QString LcpLine::host() const {
  return m_host;
}

QString LcpLine::name() const {
  return m_name;
}

LCPIO::OfflineInfo LcpLine::offlineInfo() const {
  return m_offlineInfo;
}

LCPIO::CostInfo LcpLine::costInfo() const {
  return m_costInfo;
}

bool LcpLine::useDefaultSettings() const {
  return m_useDefaultSettings;
}

void LcpLine::setStatus(int s) {
  if((m_status == CST_UP_SERVER) ||
    (m_status == CST_UP_USER) ||
    (m_status == CST_LOCKED_UP_CLT) ||
    (m_status == CST_LOCKED_UP_TIMED))
  {
    if((s != CST_UP_SERVER) ||
      (s != CST_UP_USER) ||
      (s != CST_LOCKED_UP_CLT) ||
      (s != CST_LOCKED_UP_TIMED))
    {
      s_numLinesOnline--;
    }
  } else if((m_status != CST_UP_SERVER) ||
    (m_status != CST_UP_USER) ||
    (m_status != CST_LOCKED_UP_CLT) ||
    (m_status != CST_LOCKED_UP_TIMED))
  {
    if((s == CST_UP_SERVER) ||
      (s == CST_UP_USER) ||
      (s == CST_LOCKED_UP_CLT) ||
      (s == CST_LOCKED_UP_TIMED))
    {
      s_numLinesOnline++;
    }
  }

  m_status = s;
}

void LcpLine::setMaxThroughputUp(uint m) {
  m_maxThroughputUp = m;
}

void LcpLine::setMaxThroughputDown(uint m) {
  m_maxThroughputDown = m;
}

void LcpLine::setNumChannels(int num) {
  m_maxThroughputUp = (m_maxThroughputUp/m_numChannels)*num; // set to new throughput
  m_maxThroughputDown = (m_maxThroughputDown/m_numChannels)*num; // set to new throughput
  m_numChannels = num;
}

void LcpLine::setOfflineInfo(LCPIO::OfflineInfo info) {
  m_offlineInfo = info;
}

void LcpLine::setCostInfo(LCPIO::CostInfo info) {
  m_costInfo = info;
}

void LcpLine::setUseDefaultSettings(bool use) {
  m_useDefaultSettings = use;
}

void LcpLine::setName(const QString name) {
  m_name = name;
}

void LcpLine::calculateCost(struct t_lcp3_info_tput info, double& client, double& line) {
  double fee = m_costInfo.connectionFee;
  double cost = m_costInfo.cost;

  if(m_costInfo.costCalcActive) {
    if(m_costInfo.costType == LCPIO::Time) {
      if((m_status == CST_UP_SERVER) ||
        (m_status == CST_UP_USER) ||
        (m_status == CST_LOCKED_UP_CLT) ||
        (m_status == CST_LOCKED_UP_TIMED))
      {
        client = (fee * m_numChannels) + ((static_cast<int>(info.client / 60) + 1) * cost * m_numChannels);
      } else {
        client = 0;
      }

      line = (fee * m_numChannels) + ((static_cast<int>(info.line / 60) + 1) * cost * m_numChannels);
    }
  }
}

bool LcpLine::autoOffline(struct t_lcp3_info_tput info) {
  bool time, down, up, maxDown, maxUp;
  time = down = up = maxDown = maxUp = false;

  if(m_offlineInfo.autoOfflineActive && !s_autoOfflineDisabled) {
    if(m_offlineInfo.timeActive && (info.client >= m_offlineInfo.time)) {
      time = true;
    } else if(!m_offlineInfo.timeActive && (m_offlineInfo.logic == LCPIO::AND)) {
      time = true;
    }

    if(m_offlineInfo.downloadActive && (info.dns <= m_offlineInfo.download)) {
      down = true;
    } else if(!m_offlineInfo.downloadActive && (m_offlineInfo.logic == LCPIO::AND)) {
      down = true;
    }

    if(m_offlineInfo.uploadActive && (info.ups <= m_offlineInfo.upload)) {
      up = true;
    } else if(!m_offlineInfo.uploadActive && (m_offlineInfo.logic == LCPIO::AND)) {
      up = true;
    }

    if(m_offlineInfo.maxDownloadActive && (info.dnt >= m_offlineInfo.maxDownload)) {
      maxDown = true;
    } else if(!m_offlineInfo.maxDownloadActive && (m_offlineInfo.logic == LCPIO::AND)) {
      maxDown = true;
    }

    if(m_offlineInfo.maxUploadActive && (info.upt >= m_offlineInfo.maxUpload)) {
      maxUp = true;
    } else if(!m_offlineInfo.maxUploadActive && (m_offlineInfo.logic == LCPIO::AND)) {
      maxUp = true;
    }

    if(m_offlineInfo.mode == LCPIO::Activity) {
      if(m_offlineInfo.logic == LCPIO::AND) {
        if(down && up && maxDown && maxUp) {
          if(m_activityTime == 0) {
            m_activityTime = info.client;
          }
        } else {
          m_activityTime = 0;
        }
      } else if(m_offlineInfo.logic == LCPIO::OR) {
        if(down || up || maxDown || maxUp) {
          if(m_activityTime == 0) {
            m_activityTime = info.client;
          }
        } else {
          m_activityTime = 0;
        }
      }

      if(((info.client - m_activityTime) >= m_offlineInfo.time) && (m_activityTime != 0)) {
        return true;
      } else {
        return false;
      }
    } else if(m_offlineInfo.mode == LCPIO::Normal) {
      if(m_offlineInfo.logic == LCPIO::AND) {
        if(time && down && up && maxDown && maxUp) {
          return true;
        } else {
          return false;
        }
      } else if(m_offlineInfo.logic == LCPIO::OR) {
        if(time || down || up || maxDown || maxUp) {
          return true;
        } else {
          return false;
        }
      }
    }
  }

  return false;
}

void LcpLine::loadSettings() {
  KConfig config("klclines");
  if(m_name == "Default") {
    m_useDefaultSettings = true;
  } else {
    config.setGroup(m_host + ":" + name());
    m_useDefaultSettings = config.readBoolEntry("Use Default", true);
  }

  if(m_useDefaultSettings) {
    config.setGroup("Default");
    m_offlineInfo.autoOfflineActive = config.readBoolEntry("Use Auto-offline", false);
    m_offlineInfo.timeActive = config.readBoolEntry("Use Time Limit", false);
    m_offlineInfo.time = config.readUnsignedNumEntry("Time Limit", 60);
    m_offlineInfo.downloadActive = config.readBoolEntry("Use Download Limit", false);
    m_offlineInfo.download = config.readUnsignedNumEntry("Download Limit", 1000);
    m_offlineInfo.uploadActive = config.readBoolEntry("Use Upload Limit", false);
    m_offlineInfo.upload = config.readUnsignedNumEntry("Upload Limit", 100);
    m_offlineInfo.maxDownloadActive = config.readBoolEntry("Use Max Download Limit", false);
    m_offlineInfo.maxDownload = config.readUnsignedNumEntry("Max Download Limit", 20480);
    m_offlineInfo.maxUploadActive = config.readBoolEntry("Use Max Upload Limit", false);
    m_offlineInfo.maxUpload = config.readUnsignedNumEntry("Max Upload Limit", 2048);
    m_offlineInfo.logic = static_cast<LCPIO::LogicType> (config.readNumEntry("Logic", LCPIO::OR));
    m_offlineInfo.mode = static_cast<LCPIO::ModeType> (config.readNumEntry("Mode", LCPIO::Normal));
    m_costInfo.costCalcActive = config.readBoolEntry("Use Cost Calculation", false);
    m_costInfo.cost = config.readDoubleNumEntry("Cost", 0);
    m_costInfo.connectionFee = config.readDoubleNumEntry("Connection Fee", 0);
    m_costInfo.costType = static_cast<LCPIO::CostType> (config.readNumEntry("Type", LCPIO::Time));
    m_useTwoChanDefault = config.readBoolEntry("Default To Two Channels", false);
    m_onlineScriptActive = config.readBoolEntry("Use Online Script", false);
    m_onlineScript = config.readEntry("Online Script", "");
    m_offlineScriptActive = config.readBoolEntry("Use Offline Script", false);
    m_offlineScript = config.readEntry("Offline Script", "");
  } else {
    m_offlineInfo.autoOfflineActive = config.readBoolEntry("Use Auto-offline", false);
    m_offlineInfo.timeActive = config.readBoolEntry("Use Time Limit", false);
    m_offlineInfo.time = config.readUnsignedNumEntry("Time Limit", 60);
    m_offlineInfo.downloadActive = config.readBoolEntry("Use Download Limit", false);
    m_offlineInfo.download = config.readUnsignedNumEntry("Download Limit", 1000);
    m_offlineInfo.uploadActive = config.readBoolEntry("Use Upload Limit", false);
    m_offlineInfo.upload = config.readUnsignedNumEntry("Upload Limit", 100);
    m_offlineInfo.maxDownloadActive = config.readBoolEntry("Use Max Download Limit", false);
    m_offlineInfo.maxDownload = config.readUnsignedNumEntry("Max Download Limit", 20480);
    m_offlineInfo.maxUploadActive = config.readBoolEntry("Use Max Upload Limit", false);
    m_offlineInfo.maxUpload = config.readUnsignedNumEntry("Max Upload Limit", 2048);
    m_offlineInfo.logic = static_cast<LCPIO::LogicType> (config.readNumEntry("Logic", LCPIO::OR));
    m_offlineInfo.mode = static_cast<LCPIO::ModeType> (config.readNumEntry("Mode", LCPIO::Normal));
    m_costInfo.costCalcActive = config.readBoolEntry("Use Cost Calculation", false);
    m_costInfo.cost = config.readDoubleNumEntry("Cost", 0);
    m_costInfo.connectionFee = config.readDoubleNumEntry("Connection Fee", 0);
    m_costInfo.costType = static_cast<LCPIO::CostType> (config.readNumEntry("Cost Type", LCPIO::Time));
    m_useTwoChanDefault = config.readBoolEntry("Default To Two Channels", false);
    m_onlineScriptActive = config.readBoolEntry("Use Online Script", false);
    m_onlineScript = config.readEntry("Online Script", "");
    m_offlineScriptActive = config.readBoolEntry("Use Offline Script", false);
    m_offlineScript = config.readEntry("Offline Script", "");
  }
}

void LcpLine::saveSettings() {
  KConfig config("klclines");
  if(m_name == "Default") {
    config.setGroup("Default");
    m_useDefaultSettings = false;
  } else {
    config.setGroup(m_host + ":" + name());
    config.writeEntry("Use Default", m_useDefaultSettings);
  }

  if(!m_useDefaultSettings) {
    config.writeEntry("Use Auto-offline", m_offlineInfo.autoOfflineActive);
    config.writeEntry("Use Time Limit", m_offlineInfo.timeActive);
    config.writeEntry("Time Limit", m_offlineInfo.time);
    config.writeEntry("Use Download Limit", m_offlineInfo.downloadActive);
    config.writeEntry("Download Limit", m_offlineInfo.download);
    config.writeEntry("Use Upload Limit", m_offlineInfo.uploadActive);
    config.writeEntry("Upload Limit", m_offlineInfo.upload);
    config.writeEntry("Use Max Download Limit", m_offlineInfo.maxDownloadActive);
    config.writeEntry("Max Download Limit", m_offlineInfo.maxDownload);
    config.writeEntry("Use Max Upload Limit", m_offlineInfo.maxUploadActive);
    config.writeEntry("Max Upload Limit", m_offlineInfo.maxUpload);
    config.writeEntry("Logic", m_offlineInfo.logic);
    config.writeEntry("Mode", m_offlineInfo.mode);
    config.writeEntry("Use Cost Calculation", m_costInfo.costCalcActive);
    config.writeEntry("Cost", m_costInfo.cost);
    config.writeEntry("Connection Fee", m_costInfo.connectionFee);
    config.writeEntry("Cost Type", m_costInfo.costType);
    config.writeEntry("Default To Two Channels", m_useTwoChanDefault);
    config.writeEntry("Use Online Script", m_onlineScriptActive);
    config.writeEntry("Online Script", m_onlineScript);
    config.writeEntry("Use Offline Script", m_offlineScriptActive);
    config.writeEntry("Offline Script", m_offlineScript);
  }
}

void LcpLine::setAutoOfflineDisabled(bool disable) {
  s_autoOfflineDisabled = disable;
}

bool LcpLine::autoOfflineDisabled() {
  return s_autoOfflineDisabled;
}

void LcpLine::setUseTwoChanDefault(bool use) {
  m_useTwoChanDefault = use;
}

bool LcpLine::useTwoChanDefault() const {
  return m_useTwoChanDefault;
}

void LcpLine::setOnlineScriptActive(bool active) {
  m_onlineScriptActive = active;
}

void LcpLine::setOfflineScriptActive(bool active) {
  m_offlineScriptActive = active;
}

void LcpLine::setOnlineScript(QString path) {
  m_onlineScript = path;
}

void LcpLine::setOfflineScript(QString path) {
  m_offlineScript = path;
}

bool LcpLine::onlineScriptActive() const {
  return m_onlineScriptActive;
}

bool LcpLine::offlineScriptActive() const {
  return m_offlineScriptActive;
}

QString LcpLine::onlineScript() const {
  return m_onlineScript;
}

QString LcpLine::offlineScript() const {
  return m_offlineScript;
}

void LcpLine::setLineId(int id) {
  m_lineId = id;
}

int LcpLine::lineId() const {
  return m_lineId;
}

int LcpLine::numLinesOnline() {
  return s_numLinesOnline;
}
