/***************************************************************************
                          lcpline.h  -  description
                             -------------------
    begin                : Thu Oct 18 2001
    copyright            : (C) 2001 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LCPLINE_H
#define LCPLINE_H

// app includes
#include "misc.h"

/**This class handles a LCP-line
  *@author Peter Simonsson
  */

class QString;

class LcpLine {
  public:
    LcpLine(const QString name = 0, const QString host = 0);
    ~LcpLine();
    int status() const;
    uint maxThroughputUp() const;
    uint maxThroughputDown() const;
    int numChannels() const;
    QString host() const;
    QString name() const;
    LCPIO::OfflineInfo offlineInfo() const;
    LCPIO::CostInfo costInfo() const;
    bool useDefaultSettings() const;
    /**
     * calculate cost for the line
     * @param info lineinfo needed for calculation
     * @param client calculated cost for clientconnection
     * @param line calculated cost for lineconnection
     */
    void calculateCost(struct t_lcp3_info_tput info, double& client, double& line);
    /**
     * check if we want to go offline
     * @param info lineinfo needed to deside
     */
    bool autoOffline(struct t_lcp3_info_tput info);
    /** is autOffline temp. disabled */
    static bool autoOfflineDisabled();
    bool useTwoChanDefault() const;
    void setStatus(int s);
    void setMaxThroughputUp(uint m);
    void setMaxThroughputDown(uint m);
    void setOfflineInfo(LCPIO::OfflineInfo info);
    void setCostInfo(LCPIO::CostInfo info);
    void setNumChannels(int num);
    void setUseDefaultSettings(bool use);
    static void setAutoOfflineDisabled(bool disable);
    void setUseTwoChanDefault(bool use);
    void setName(const QString name);
    void loadSettings();
    void saveSettings();
    void setOnlineScriptActive(bool active);
    void setOfflineScriptActive(bool active);
    void setOnlineScript(QString path);
    void setOfflineScript(QString path);
    bool onlineScriptActive() const;
    bool offlineScriptActive() const;
    QString onlineScript() const;
    QString offlineScript() const;
    void setLineId(int id);
    int lineId() const;
    static int numLinesOnline();
  protected: // VARIABLES //
    QString m_host;
    QString m_name;
    int m_lineId;
    int m_status;
    uint m_maxThroughputUp;
    uint m_maxThroughputDown;
    int m_numChannels;
    LCPIO::OfflineInfo m_offlineInfo;
    LCPIO::CostInfo m_costInfo;
    bool m_useDefaultSettings;
    int m_activityTime;
    bool m_useTwoChanDefault;
    bool m_onlineScriptActive;
    bool m_offlineScriptActive;
    QString m_onlineScript;
    QString m_offlineScript;
  private:
    static bool s_autoOfflineDisabled;
    static int s_numLinesOnline;
};

#endif
