/***************************************************************************
                          sendqpost.cpp  -  description
                             -------------------
    begin                : Mon Feb 12 2001
    copyright            : (C) 2001 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Standard headers
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

// KDE headers
#include <kdebug.h>
#include <kextsock.h>

// App headers
#include "sendqpost.h"
#include "lcp3.h"

SendQPost::SendQPost(int cmd, void* info, int infoSize) {
  this->cmd = cmd;
  this->info = info;
  this->infoSize = infoSize;
  this->prevSend = 0;
  this->sendCount = 0;
}

bool SendQPost::send(KExtendedSocket* socket, int lineId) {
  bool success = true;

  //kdDebug() << "Sending " << this->cmd << "..." << endl;
  // Size of packet to send
  int packetSize;
  // Buffer for packet creation
  char* buffer = (char*) malloc(get_lcp3_pack_size(this->cmd));
  // Pointer to the cmd part of buffer
  struct t_lcp3_cmd* bufferCmd = (struct t_lcp3_cmd*)buffer;
  // Pointer to the info part of buffer
  char* bufferInfo = ((char*)buffer) + sizeof(struct t_lcp3_cmd);

  bufferCmd->magic = LCP3_MAGICNR;
  bufferCmd->version = LCP3_PROTO_VERSION;
  bufferCmd->cmd = this->cmd;
  bufferCmd->line_id = lineId;

  // Add info part if it exists
  if(this->info != NULL && this->infoSize != 0) {
    memcpy(bufferInfo, this->info, this->infoSize);
  }

  packetSize = sizeof(struct t_lcp3_cmd) + this->infoSize;

  if(socket->writeBlock(buffer, packetSize) < 0) {
    success = false;
    //kdDebug() << "Failed!" << endl;
  } else {
    //kdDebug() << "Ok" << endl;
  }

  this->prevSend = time(NULL);
  this->sendCount++;
  free(buffer);

  return success;
}

bool SendQPost::isTimedOut() const {
  return ((this->prevSend < (time(NULL) - LCP3_TMO_ACK)) && this->sendCount) ? true : false;
}

bool SendQPost::noTryLeft() const {
  return (this->sendCount >= LCP3_TMO_CNT);
}

int SendQPost::getCmd() const {
  return this->cmd;
}
